/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.company.rest;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.company.domain.DmCompany;
import me.zhengjie.modules.dm.company.service.DmCompanyService;
import me.zhengjie.modules.dm.company.service.dto.DmCompanyQueryCriteria;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

/**
* @website https://el-admin.vip
* @author shiguangji
* @date 2021-03-11
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "单位管理")
@RequestMapping("/api/dmCompany")
public class DmCompanyController {

    private final DmCompanyService dmCompanyService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmCompany:list')")
    public void download(HttpServletResponse response, DmCompanyQueryCriteria criteria) throws IOException {
        dmCompanyService.download(dmCompanyService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询单位")
    @ApiOperation("查询单位")
    @PreAuthorize("@el.check('dmCompany:list')")
    public ResponseEntity<Object> query(DmCompanyQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmCompanyService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增单位")
    @ApiOperation("新增单位")
    @PreAuthorize("@el.check('dmCompany:add')")                                     //, @RequestParam("file") MultipartFile file
    public ResponseEntity<Object> create(@Validated @RequestBody DmCompany resources){
        return new ResponseEntity<>(dmCompanyService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改单位")
    @ApiOperation("修改单位")
    @PreAuthorize("@el.check('dmCompany:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmCompany resources){
        dmCompanyService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除单位")
    @ApiOperation("删除单位")
    @PreAuthorize("@el.check('dmCompany:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        Map<String,Object> map = dmCompanyService.deleteAll(ids);
        return new ResponseEntity<>(map ,HttpStatus.OK);
    }
}
