/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.company.service;

import me.zhengjie.modules.dm.company.domain.DmCompany;
import me.zhengjie.modules.dm.company.service.dto.DmCompanyDto;
import me.zhengjie.modules.dm.company.service.dto.DmCompanyQueryCriteria;
import org.springframework.data.domain.Pageable;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author shiguangji
* @date 2021-03-11
**/
public interface DmCompanyService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmCompanyQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmCompanyDto>
    */
    List<DmCompanyDto> queryAll(DmCompanyQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param companyId ID
     * @return DmCompanyDto
     */
    DmCompanyDto findById(Long companyId);

    /**
    * 创建
    * @param resources /
    * @return DmCompanyDto
    */
    DmCompanyDto create(DmCompany resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmCompany resources);

    /**
    * 多选删除
    * @param ids /
    */
    Map<String,Object> deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmCompanyDto> all, HttpServletResponse response) throws IOException;
}