/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmCard.rest;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmCard.domain.DmCard;
import me.zhengjie.modules.dm.dmCard.service.DmCardService;
import me.zhengjie.modules.dm.dmCard.service.dto.DmCardQueryCriteria;
import me.zhengjie.modules.quartz.utils.CardUntils;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2021-11-10
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "card管理")
@RequestMapping("/api/dmCard")
public class DmCardController {

    private final DmCardService dmCardService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmCard:list')")
    public void download(HttpServletResponse response, DmCardQueryCriteria criteria) throws IOException {
        dmCardService.download(dmCardService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询card")
    @ApiOperation("查询card")
    @PreAuthorize("@el.check('dmCard:list')")
    public ResponseEntity<Object> query(DmCardQueryCriteria criteria, Pageable pageable){
         Map<String, Object> map = dmCardService.queryAll(criteria, pageable);
        return new ResponseEntity<>(map,HttpStatus.OK);
    }

    @GetMapping(value = "getCardBalance")
    @Log("查询card余额")
    @ApiOperation("查询card余额")
    @PreAuthorize("@el.check('dmCard:list')")
    public ResponseEntity<Object> getCardBalance(@RequestParam(value = "cardId") String cardId){
         JSONObject cardBalance = CardUntils.getCardBalance(cardId);
        return new ResponseEntity<>(cardBalance,HttpStatus.OK);
    }

    @PostMapping
    @Log("新增card")
    @ApiOperation("新增card")
    @PreAuthorize("@el.check('dmCard:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmCard resources) throws Exception {
        return new ResponseEntity<>(dmCardService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改card")
    @ApiOperation("修改card")
    @PreAuthorize("@el.check('dmCard:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmCard resources) throws Exception {
        dmCardService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除card")
    @ApiOperation("删除card")
    @PreAuthorize("@el.check('dmCard:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmCardService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping(value = "/cardAll")
    @Log("查询全部card")
    @ApiOperation("查询全部card")
    @PreAuthorize("@el.check('dmCard:list')")
    public ResponseEntity<Object> cardAll(){
        return new ResponseEntity<>(dmCardService.getAllByCardNumber(),HttpStatus.OK);
    }
}