/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmCard.service;

import me.zhengjie.modules.dm.dmCard.domain.DmCard;
import me.zhengjie.modules.dm.dmCard.service.dto.DmCardDto;
import me.zhengjie.modules.dm.dmCard.service.dto.DmCardQueryCriteria;
import org.springframework.data.domain.Pageable;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author zhirong
* @date 2021-11-10
**/
public interface DmCardService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmCardQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmCardDto>
    */
    List<DmCardDto> queryAll(DmCardQueryCriteria criteria);

    List<Map<String,Object>> getAllByCardNumber();

    /**
     * 根据ID查询
     * @param userName ID
     * @return DmCardDto
     */
    DmCardDto findById(Long userName);

    /**
    * 创建
    * @param resources /
    * @return DmCardDto
    */
    DmCardDto create(DmCard resources) throws Exception;

    /**
    * 编辑
    * @param resources /
    */
    void update(DmCard resources) throws Exception;

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmCardDto> all, HttpServletResponse response) throws IOException;

    List<DmCard> findByDmUserId(String userNumber, Long cardId);

    /**
     * 根据卡号查询
     * @param cardNumber 卡号
     * @return DmCard
     */
    DmCard findByCardNumber(String cardNumber);

    /**
     * 根据用户id查询
     * @param userNumber userid
     * @return DmCard
     */
    DmCard findByUserNumber(String userNumber);

}