/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmPlateTraffic.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmPlateTraffic.domain.DmPlateTraffic;
import me.zhengjie.modules.dm.dmPlateTraffic.service.DmPlateTrafficService;
import me.zhengjie.modules.dm.dmPlateTraffic.service.dto.DmPlateTrafficQueryCriteria;
import me.zhengjie.modules.system.service.UserService;
import me.zhengjie.modules.system.service.dto.RoleSmallDto;
import me.zhengjie.modules.system.service.dto.UserDto;
import me.zhengjie.utils.SecurityUtils;
import me.zhengjie.ykt.domain.DmPlateNumber;
import me.zhengjie.ykt.service.DmPlateNumberService;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author juan
* @date 2021-11-23
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/dmPlateTraffic管理")
@RequestMapping("/api/dmPlateTraffic")
public class DmPlateTrafficController {

    private final DmPlateTrafficService dmPlateTrafficService;
    private final UserService userService;
    private final DmPlateNumberService dmPlateNumberService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmPlateTraffic:list')")
    public void download(HttpServletResponse response, DmPlateTrafficQueryCriteria criteria) throws IOException {
        dmPlateTrafficService.download(dmPlateTrafficService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询api/dmPlateTraffic")
    @ApiOperation("查询api/dmPlateTraffic")
    @PreAuthorize("@el.check('dmPlateTraffic:list')")
    public ResponseEntity<Object> query(DmPlateTrafficQueryCriteria criteria, Pageable pageable){
        UserDto userDto = userService.findByName(SecurityUtils.getCurrentUsername());
        List<Integer> levels = new ArrayList<>();
        for(RoleSmallDto roleSmallDto : userDto.getRoles()){
            levels.add(roleSmallDto.getLevel());
        }
        if(!levels.contains(1)){
            DmPlateNumber dmPlateNumber = dmPlateNumberService.findByUserId(userDto.getId());
            criteria.setPlateNumber(dmPlateNumber.getPlateNumber());
        }
        return new ResponseEntity<>(dmPlateTrafficService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/dmPlateTraffic")
    @ApiOperation("新增api/dmPlateTraffic")
    @PreAuthorize("@el.check('dmPlateTraffic:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmPlateTraffic resources){
        return new ResponseEntity<>(dmPlateTrafficService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/dmPlateTraffic")
    @ApiOperation("修改api/dmPlateTraffic")
    @PreAuthorize("@el.check('dmPlateTraffic:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmPlateTraffic resources){
        dmPlateTrafficService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/dmPlateTraffic")
    @ApiOperation("删除api/dmPlateTraffic")
    @PreAuthorize("@el.check('dmPlateTraffic:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmPlateTrafficService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}