/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmReadingRoom.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmReadingRoom.domain.DmReadingRoom;
import me.zhengjie.modules.dm.dmReadingRoom.service.DmReadingRoomService;
import me.zhengjie.modules.dm.dmReadingRoom.service.dto.DmReadingRoomQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author longda
* @date 2022-02-25
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/readRoom管理")
@RequestMapping("/api/dmReadingRoom")
public class DmReadingRoomController {

    private final DmReadingRoomService dmReadingRoomService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmReadingRoom:list')")
    public void download(HttpServletResponse response, DmReadingRoomQueryCriteria criteria) throws IOException {
        dmReadingRoomService.download(dmReadingRoomService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/readRoom")
    @ApiOperation("查询/api/readRoom")
    @PreAuthorize("@el.check('dmReadingRoom:list')")
    public ResponseEntity<Object> query(DmReadingRoomQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmReadingRoomService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/readRoom")
    @ApiOperation("新增/api/readRoom")
    @PreAuthorize("@el.check('dmReadingRoom:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmReadingRoom resources){
        return new ResponseEntity<>(dmReadingRoomService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/readRoom")
    @ApiOperation("修改/api/readRoom")
    @PreAuthorize("@el.check('dmReadingRoom:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmReadingRoom resources){
        dmReadingRoomService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/readRoom")
    @ApiOperation("删除/api/readRoom")
    @PreAuthorize("@el.check('dmReadingRoom:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmReadingRoomService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}