/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmReadingRoomApplay.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmReadingRoomApplay.domain.DmReadingRoomApplay;
import me.zhengjie.modules.dm.dmReadingRoomApplay.service.DmReadingRoomApplayService;
import me.zhengjie.modules.dm.dmReadingRoomApplay.service.dto.DmReadingRoomApplayQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author longda
* @date 2022-03-01
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/readRoomApplay管理")
@RequestMapping("/api/dmReadingRoomApplay")
public class DmReadingRoomApplayController {

    private final DmReadingRoomApplayService dmReadingRoomApplayService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmReadingRoomApplay:list')")
    public void download(HttpServletResponse response, DmReadingRoomApplayQueryCriteria criteria) throws IOException {
        dmReadingRoomApplayService.download(dmReadingRoomApplayService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/readRoomApplay")
    @ApiOperation("查询/api/readRoomApplay")
    @PreAuthorize("@el.check('dmReadingRoomApplay:list')")
    public ResponseEntity<Object> query(DmReadingRoomApplayQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmReadingRoomApplayService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @GetMapping(value = "/queryById")
    @Log("查询/api/readRoomApplay")
    @ApiOperation("查询/api/readRoomApplay")
    @PreAuthorize("@el.check('dmReadingRoomApplay:list')")
    public ResponseEntity<Object> queryById(@RequestParam(value = "roomId") Long roomId){
        return new ResponseEntity<>(dmReadingRoomApplayService.findApplayByRoomId(roomId),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/readRoomApplay")
    @ApiOperation("新增/api/readRoomApplay")
    @PreAuthorize("@el.check('dmReadingRoomApplay:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmReadingRoomApplay resources){
        return new ResponseEntity<>(dmReadingRoomApplayService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/readRoomApplay")
    @ApiOperation("修改/api/readRoomApplay")
    @PreAuthorize("@el.check('dmReadingRoomApplay:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmReadingRoomApplay resources){
        dmReadingRoomApplayService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/readRoomApplay")
    @ApiOperation("删除/api/readRoomApplay")
    @PreAuthorize("@el.check('dmReadingRoomApplay:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody String[] ids) {
        dmReadingRoomApplayService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}