/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmReadingRoomApplay.service;

import me.zhengjie.modules.dm.dmReadingRoomApplay.domain.DmReadingRoomApplay;
import me.zhengjie.modules.dm.dmReadingRoomApplay.service.dto.DmReadingRoomApplayDto;
import me.zhengjie.modules.dm.dmReadingRoomApplay.service.dto.DmReadingRoomApplayQueryCriteria;
import org.springframework.data.domain.Pageable;

import java.math.BigDecimal;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author longda
* @date 2022-03-01
**/
public interface DmReadingRoomApplayService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmReadingRoomApplayQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmReadingRoomApplayDto>
    */
    List<DmReadingRoomApplayDto> queryAll(DmReadingRoomApplayQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param readomgRoomApplayId ID
     * @return DmReadingRoomApplayDto
     */
    DmReadingRoomApplayDto findById(String readomgRoomApplayId);

    /**
    * 创建
    * @param resources /
    * @return DmReadingRoomApplayDto
    */
    DmReadingRoomApplayDto create(DmReadingRoomApplay resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmReadingRoomApplay resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(String[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmReadingRoomApplayDto> all, HttpServletResponse response) throws IOException;

    /**
     * 查询消费总金额
     * @param readingRoomId  消费记录id
     * @return
     */
    BigDecimal findApplaySumMoneyByRoomId(Long readingRoomId);
    /**
     * 查询消今日费记录
     * @param readingRoomId  消费记录id
     * @return
     */
    List<DmReadingRoomApplay> findApplayNowByRoomId(Long readingRoomId);

    /**
     * 根据阅览id查询消费记录
     * @param roomId
     * @return
     */
    List<DmReadingRoomApplay> findApplayByRoomId(Long roomId);
}