/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmReadingRoomConfigOverdue.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmReadingRoomConfigOverdue.domain.DmReadingRoomConfigOverdue;
import me.zhengjie.modules.dm.dmReadingRoomConfigOverdue.service.DmReadingRoomConfigOverdueService;
import me.zhengjie.modules.dm.dmReadingRoomConfigOverdue.service.dto.DmReadingRoomConfigOverdueQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author longda
* @date 2022-03-02
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/configOverdue管理")
@RequestMapping("/api/dmReadingRoomConfigOverdue")
public class DmReadingRoomConfigOverdueController {

    private final DmReadingRoomConfigOverdueService dmReadingRoomConfigOverdueService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmReadingRoomConfigOverdue:list')")
    public void download(HttpServletResponse response, DmReadingRoomConfigOverdueQueryCriteria criteria) throws IOException {
        dmReadingRoomConfigOverdueService.download(dmReadingRoomConfigOverdueService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/configOverdue")
    @ApiOperation("查询/api/configOverdue")
    @PreAuthorize("@el.check('dmReadingRoomConfigOverdue:list')")
    public ResponseEntity<Object> query(DmReadingRoomConfigOverdueQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmReadingRoomConfigOverdueService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/configOverdue")
    @ApiOperation("新增/api/configOverdue")
    @PreAuthorize("@el.check('dmReadingRoomConfigOverdue:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmReadingRoomConfigOverdue resources){
        return new ResponseEntity<>(dmReadingRoomConfigOverdueService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/configOverdue")
    @ApiOperation("修改/api/configOverdue")
    @PreAuthorize("@el.check('dmReadingRoomConfigOverdue:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmReadingRoomConfigOverdue resources){
        dmReadingRoomConfigOverdueService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/configOverdue")
    @ApiOperation("删除/api/configOverdue")
    @PreAuthorize("@el.check('dmReadingRoomConfigOverdue:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody String[] ids) {
        dmReadingRoomConfigOverdueService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}