/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmVisitorsApply.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmVisitorsApply.domain.DmVisitorsApply;
import me.zhengjie.modules.dm.dmVisitorsApply.service.DmVisitorsApplyService;
import me.zhengjie.modules.dm.dmVisitorsApply.service.dto.DmVisitorsApplyQueryCriteria;
import me.zhengjie.modules.dm.dmVisitorsApply.service.dto.DmVisitorsDto;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author juan
* @date 2021-11-19
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/dmVisitorsApply管理")
@RequestMapping("/api/dmVisitorsApply")
public class DmVisitorsApplyController {

    private final DmVisitorsApplyService dmVisitorsApplyService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmVisitorsApply:list')")
    public void download(HttpServletResponse response, DmVisitorsApplyQueryCriteria criteria) throws IOException {
        dmVisitorsApplyService.download(dmVisitorsApplyService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询api/dmVisitorsApply")
    @ApiOperation("查询api/dmVisitorsApply")
    @PreAuthorize("@el.check('dmVisitorsApply:list')")
    public ResponseEntity<Object> query(DmVisitorsApplyQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmVisitorsApplyService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/dmVisitorsApply")
    @ApiOperation("新增api/dmVisitorsApply")
    @PreAuthorize("@el.check('dmVisitorsApply:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmVisitorsDto resources){
        return new ResponseEntity<>(dmVisitorsApplyService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/dmVisitorsApply")
    @ApiOperation("修改api/dmVisitorsApply")
    @PreAuthorize("@el.check('dmVisitorsApply:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmVisitorsDto resources){
        dmVisitorsApplyService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/dmVisitorsApply")
    @ApiOperation("删除api/dmVisitorsApply")
    @PreAuthorize("@el.check('dmVisitorsApply:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmVisitorsApplyService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}