/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmVisitorsUser.service;

import com.alibaba.fastjson.JSONObject;
import me.zhengjie.modules.dm.dmVisitorsUser.domain.DmVisitorsUser;
import me.zhengjie.modules.dm.dmVisitorsUser.service.dto.DmVisitorsUserDto;
import me.zhengjie.modules.dm.dmVisitorsUser.service.dto.DmVisitorsUserQueryCriteria;
import org.springframework.data.domain.Pageable;

import java.text.ParseException;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author juan
* @date 2021-11-19
**/
public interface DmVisitorsUserService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmVisitorsUserQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmVisitorsUserDto>
    */
    List<DmVisitorsUserDto> queryAll(DmVisitorsUserQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param visitorId ID
     * @return DmVisitorsUserDto
     */
    DmVisitorsUserDto findById(Long visitorId);

    /**
    * 创建
    * @param resources /
    * @return DmVisitorsUserDto
    */
    DmVisitorsUserDto create(DmVisitorsUser resources);

    /**
     * 创建
     * @param resources /
     */
    void createAll(List<DmVisitorsUser> resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmVisitorsUser resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmVisitorsUserDto> all, HttpServletResponse response) throws IOException;

    /**
     * 接收车辆进出场记录
     * @param json json
     */
    void vehicleAccessTraffic(JSONObject json) throws ParseException;

    /**
     * 根据被访客id查询所有访客
     * @param visitorsApplyId 条件参数
     * @return List<DmVisitorsUserDto>
     */
    List<DmVisitorsUser> findByApplyId(Long visitorsApplyId);

}