/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmtsCollection.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmtsCollection.domain.DmTsCollection;
import me.zhengjie.modules.dm.dmtsCollection.service.DmTsCollectionService;
import me.zhengjie.modules.dm.dmtsCollection.service.dto.DmTsCollectionQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author longda
* @date 2022-03-01
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/tsCollection管理")
@RequestMapping("/api/dmTsCollection")
public class DmTsCollectionController {

    private final DmTsCollectionService dmTsCollectionService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmTsCollection:list')")
    public void download(HttpServletResponse response, DmTsCollectionQueryCriteria criteria) throws IOException {
        dmTsCollectionService.download(dmTsCollectionService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/tsCollection")
    @ApiOperation("查询/api/tsCollection")
    @PreAuthorize("@el.check('dmTsCollection:list')")
    public ResponseEntity<Object> query(DmTsCollectionQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmTsCollectionService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/tsCollection")
    @ApiOperation("新增/api/tsCollection")
    @PreAuthorize("@el.check('dmTsCollection:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmTsCollection resources){
        return new ResponseEntity<>(dmTsCollectionService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/tsCollection")
    @ApiOperation("修改/api/tsCollection")
    @PreAuthorize("@el.check('dmTsCollection:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmTsCollection resources){
        dmTsCollectionService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/tsCollection")
    @ApiOperation("删除/api/tsCollection")
    @PreAuthorize("@el.check('dmTsCollection:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody String[] ids) {
        dmTsCollectionService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}