/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.dmtsInfo.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.dmtsInfo.domain.DmTsInfo;
import me.zhengjie.modules.dm.dmtsInfo.service.DmTsInfoService;
import me.zhengjie.modules.dm.dmtsInfo.service.dto.DmTsInfoQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author longda
* @date 2022-03-01
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/tsInfo管理")
@RequestMapping("/api/dmTsInfo")
public class DmTsInfoController {

    private final DmTsInfoService dmTsInfoService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmTsInfo:list')")
    public void download(HttpServletResponse response, DmTsInfoQueryCriteria criteria) throws IOException {
        dmTsInfoService.download(dmTsInfoService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/tsInfo")
    @ApiOperation("查询/api/tsInfo")
    @PreAuthorize("@el.check('dmTsInfo:list')")
    public ResponseEntity<Object> query(DmTsInfoQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmTsInfoService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/tsInfo")
    @ApiOperation("新增/api/tsInfo")
    @PreAuthorize("@el.check('dmTsInfo:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmTsInfo resources){
        return new ResponseEntity<>(dmTsInfoService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/tsInfo")
    @ApiOperation("修改/api/tsInfo")
    @PreAuthorize("@el.check('dmTsInfo:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmTsInfo resources){
        dmTsInfoService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/tsInfo")
    @ApiOperation("删除/api/tsInfo")
    @PreAuthorize("@el.check('dmTsInfo:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmTsInfoService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}