/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmConsumeDevices.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dmConsumeDevices.domain.DmConsumeDevices;
import me.zhengjie.modules.dmConsumeDevices.service.DmConsumeDevicesService;
import me.zhengjie.modules.dmConsumeDevices.service.dto.DmConsumeDevicesQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2021-11-19
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/dmConsume管理")
@RequestMapping("/api/dmConsume")
public class DmConsumeDevicesController {

    private final DmConsumeDevicesService dmConsumeService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmConsume:list')")
    public void download(HttpServletResponse response, DmConsumeDevicesQueryCriteria criteria) throws IOException {
        dmConsumeService.download(dmConsumeService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询api/dmConsume")
    @ApiOperation("查询api/dmConsume")
    @PreAuthorize("@el.check('dmConsume:list')")
    public ResponseEntity<Object> query(DmConsumeDevicesQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmConsumeService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/dmConsume")
    @ApiOperation("新增api/dmConsume")
    @PreAuthorize("@el.check('dmConsume:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmConsumeDevices resources){
        return new ResponseEntity<>(dmConsumeService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/dmConsume")
    @ApiOperation("修改api/dmConsume")
    @PreAuthorize("@el.check('dmConsume:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmConsumeDevices resources){
        dmConsumeService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/dmConsume")
    @ApiOperation("删除api/dmConsume")
    @PreAuthorize("@el.check('dmConsume:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmConsumeService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}