/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmConsumeDevices.service;

import me.zhengjie.modules.dmConsumeDevices.domain.DmConsumeDevices;
import me.zhengjie.modules.dmConsumeDevices.service.dto.DmConsumeDevicesDto;
import me.zhengjie.modules.dmConsumeDevices.service.dto.DmConsumeDevicesQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author zhirong
* @date 2021-11-19
**/
public interface DmConsumeDevicesService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmConsumeDevicesQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmConsumeDto>
    */
    List<DmConsumeDevicesDto> queryAll(DmConsumeDevicesQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param consumeId ID
     * @return DmConsumeDto
     */
    DmConsumeDevicesDto findById(Long consumeId);

    /**
    * 创建
    * @param resources /
    * @return DmConsumeDto
    */
    DmConsumeDevicesDto create(DmConsumeDevices resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmConsumeDevices resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmConsumeDevicesDto> all, HttpServletResponse response) throws IOException;
}