/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmExpenseCalendar.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dmExpenseCalendar.domain.DmExpenseCalendar;
import me.zhengjie.modules.dmExpenseCalendar.service.DmExpenseCalendarService;
import me.zhengjie.modules.dmExpenseCalendar.service.dto.DmExpenseCalendarQueryCriteria;
import me.zhengjie.modules.system.domain.Role;
import me.zhengjie.modules.system.service.UserService;
import me.zhengjie.modules.system.service.dto.RoleDto;
import me.zhengjie.modules.system.service.dto.RoleSmallDto;
import me.zhengjie.modules.system.service.dto.UserDto;
import me.zhengjie.utils.SecurityUtils;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2021-11-23
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "/api/dmExpenseCalendar管理")
@RequestMapping("/api/dmExpenseCalendar")
public class DmExpenseCalendarController {

    private final DmExpenseCalendarService dmExpenseCalendarService;

    private final UserService userService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmExpenseCalendar:list')")
    public void download(HttpServletResponse response, DmExpenseCalendarQueryCriteria criteria) throws IOException {
        dmExpenseCalendarService.download(dmExpenseCalendarService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询/api/dmExpenseCalendar")
    @ApiOperation("查询/api/dmExpenseCalendar")
    @PreAuthorize("@el.check('dmExpenseCalendar:list')")
    public ResponseEntity<Object> query(DmExpenseCalendarQueryCriteria criteria, Pageable pageable){
        UserDto userDto = userService.findByName(SecurityUtils.getCurrentUsername());
        List<Integer> levels = new ArrayList<>();
        for(RoleSmallDto roleSmallDto : userDto.getRoles()){
            levels.add(roleSmallDto.getLevel());
        }
        if(!levels.contains(1)){
            criteria.setCardNumber(userDto.getDmUser().getCardId());
        }
        return new ResponseEntity<>(dmExpenseCalendarService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @GetMapping("/getAll")
    @Log("查询/api/dmExpenseCalendar")
    @ApiOperation("查询/api/dmExpenseCalendar")
    @PreAuthorize("@el.check('dmExpenseCalendar:list')")
    public ResponseEntity<Object> queryAll(DmExpenseCalendarQueryCriteria criteria){
        return new ResponseEntity<>(dmExpenseCalendarService.queryAll(criteria),HttpStatus.OK);
    }

    @GetMapping(value = "/homePageYktExpense")
    @Log("首页今日消费情况")
    @ApiOperation("首页今日消费情况")
    public ResponseEntity<Object> homePageYktExpense(){
        return new ResponseEntity<>(dmExpenseCalendarService.homePageYktExpense(),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增/api/dmExpenseCalendar")
    @ApiOperation("新增/api/dmExpenseCalendar")
    @PreAuthorize("@el.check('dmExpenseCalendar:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmExpenseCalendar resources){
        return new ResponseEntity<>(dmExpenseCalendarService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改/api/dmExpenseCalendar")
    @ApiOperation("修改/api/dmExpenseCalendar")
    @PreAuthorize("@el.check('dmExpenseCalendar:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmExpenseCalendar resources){
        dmExpenseCalendarService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除/api/dmExpenseCalendar")
    @ApiOperation("删除/api/dmExpenseCalendar")
    @PreAuthorize("@el.check('dmExpenseCalendar:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmExpenseCalendarService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}