/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmParkingTraffic.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dmParkingTraffic.domain.DmParkingTraffic;
import me.zhengjie.modules.dmParkingTraffic.service.DmParkingTrafficService;
import me.zhengjie.modules.dmParkingTraffic.service.dto.DmParkingTrafficQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2022-05-05
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/dmParkingTraffic管理")
@RequestMapping("/api/dmParkingTraffic")
public class DmParkingTrafficController {

    private final DmParkingTrafficService dmParkingTrafficService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmParkingTraffic:list')")
    public void download(HttpServletResponse response, DmParkingTrafficQueryCriteria criteria) throws IOException {
        dmParkingTrafficService.download(dmParkingTrafficService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询api/dmParkingTraffic")
    @ApiOperation("查询api/dmParkingTraffic")
    @PreAuthorize("@el.check('dmParkingTraffic:list')")
    public ResponseEntity<Object> query(DmParkingTrafficQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmParkingTrafficService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/dmParkingTraffic")
    @ApiOperation("新增api/dmParkingTraffic")
    @PreAuthorize("@el.check('dmParkingTraffic:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmParkingTraffic resources){
        return new ResponseEntity<>(dmParkingTrafficService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/dmParkingTraffic")
    @ApiOperation("修改api/dmParkingTraffic")
    @PreAuthorize("@el.check('dmParkingTraffic:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmParkingTraffic resources){
        dmParkingTrafficService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/dmParkingTraffic")
    @ApiOperation("删除api/dmParkingTraffic")
    @PreAuthorize("@el.check('dmParkingTraffic:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Integer[] ids) {
        dmParkingTrafficService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}