/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmRechargeRecord.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dmRechargeRecord.domain.DmRechargeRecord;
import me.zhengjie.modules.dmRechargeRecord.service.DmRechargeRecordService;
import me.zhengjie.modules.dmRechargeRecord.service.dto.DmRechargeRecordQueryCriteria;
import me.zhengjie.modules.system.service.UserService;
import me.zhengjie.modules.system.service.dto.RoleSmallDto;
import me.zhengjie.modules.system.service.dto.UserDto;
import me.zhengjie.utils.SecurityUtils;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2021-11-23
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/dmRechargeRecord管理")
@RequestMapping("/api/dmRechargeRecord")
public class DmRechargeRecordController {

    private final DmRechargeRecordService dmRechargeRecordService;
    private final UserService userService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmRechargeRecord:list')")
    public void download(HttpServletResponse response, DmRechargeRecordQueryCriteria criteria) throws IOException {
        dmRechargeRecordService.download(dmRechargeRecordService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询api/dmRechargeRecord")
    @ApiOperation("查询api/dmRechargeRecord")
    @PreAuthorize("@el.check('dmRechargeRecord:list')")
    public ResponseEntity<Object> query(DmRechargeRecordQueryCriteria criteria, Pageable pageable){
        UserDto userDto = userService.findByName(SecurityUtils.getCurrentUsername());
        List<Integer> levels = new ArrayList<>();
        for(RoleSmallDto roleSmallDto : userDto.getRoles()){
            levels.add(roleSmallDto.getLevel());
        }
        if(!levels.contains(1)){
            criteria.setCardNumber(userDto.getDmUser().getCardId());
        }
        return new ResponseEntity<>(dmRechargeRecordService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @GetMapping("/getAll")
    @Log("查询api/dmRechargeRecord不分页")
    @ApiOperation("查询api/dmRechargeRecord不分页")
    @PreAuthorize("@el.check('dmRechargeRecord:list')")
    public ResponseEntity<Object> queryAll(DmRechargeRecordQueryCriteria criteria){
        return new ResponseEntity<>(dmRechargeRecordService.queryAll(criteria),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/dmRechargeRecord")
    @ApiOperation("新增api/dmRechargeRecord")
    @PreAuthorize("@el.check('dmRechargeRecord:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmRechargeRecord resources){
        return new ResponseEntity<>(dmRechargeRecordService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/dmRechargeRecord")
    @ApiOperation("修改api/dmRechargeRecord")
    @PreAuthorize("@el.check('dmRechargeRecord:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmRechargeRecord resources){
        dmRechargeRecordService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/dmRechargeRecord")
    @ApiOperation("删除api/dmRechargeRecord")
    @PreAuthorize("@el.check('dmRechargeRecord:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmRechargeRecordService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}