/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dmuser.service;

import me.zhengjie.modules.dmuser.domain.DmUser;
import me.zhengjie.modules.dmuser.domain.Vo.DmUserVo;
import me.zhengjie.modules.dmuser.service.dto.DmUserDto;
import me.zhengjie.modules.dmuser.service.dto.DmUserQueryCriteria;
import org.apache.http.client.ClientProtocolException;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.query.Param;

import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author AndyCheung
* @date 2021-09-30
**/
public interface DmUserService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmUserQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmUserDto>
    */
    List<DmUserDto> queryAll(DmUserQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmUserDto
     */
    DmUserDto findById(String id);

    /**
    * 创建
    * @param resources /
    * @return DmUserDto
    */
    DmUserDto create(DmUser resources);

    void createUserDevice(DmUserVo dmUserVo);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmUser resources) throws Exception;

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(String[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmUserDto> all, HttpServletResponse response) throws IOException;

    /**
     * 查询未绑定账号的用户
     * @param userId
     * @return
     */
    List<DmUser> findNotBindUser(String userId);

    List<Map<String,String>> findNotUserCard(String userId);

    Map<String, Object> homePageYktUser();

    List<Map<String,Object>> findUser();

    List<Map<String,Object>> findUserByDeptId(String deptId);

    /**
     * 根据ID查询
     * @param userNumber ID
     * @return DmUserDto
     */
    DmUser findByUserNumber(String userNumber);

    /**
     * 根据userNuber查询
     * @param userNumber userNumber
     * @return DmUserDto
     */
    DmUser findByUserCardId(String userNumber);

    DmUser findByCardId(String userNumber);
}