/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.ykt.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.ykt.domain.DmRoomDevice;
import me.zhengjie.ykt.service.DmRoomDeviceService;
import me.zhengjie.ykt.service.dto.DmRoomDeviceQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author zhirong
* @date 2021-11-19
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "api/ykt/room_device管理")
@RequestMapping("/api/dmRoomDevice")
public class DmRoomDeviceController {

    private final DmRoomDeviceService dmRoomDeviceService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmRoomDevice:list')")
    public void download(HttpServletResponse response, DmRoomDeviceQueryCriteria criteria) throws IOException {
        dmRoomDeviceService.download(dmRoomDeviceService.queryAll(criteria), response);
    }


    @Log("导入Excel数据")
    @ApiOperation("导入Excel数据")
    @PostMapping(value = "/importData")
    @PreAuthorize("@el.check('dmAccessTraffic:list')")
    public Map<String, Object> importData(@RequestParam("file") MultipartFile file) {
        List<Map<Object, Object>> list = dmRoomDeviceService.importExcel(file);
        Map<String, Object> map = new HashMap<>();
        map.put("content", list);
        map.put("totalElement", list.size());
        return map;
    }

    @GetMapping
    @Log("查询api/ykt/room_device")
    @ApiOperation("查询api/ykt/room_device")
    @PreAuthorize("@el.check('dmRoomDevice:list')")
    public ResponseEntity<Object> query(DmRoomDeviceQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmRoomDeviceService.queryAll(criteria,pageable),HttpStatus.OK);
    }
    @GetMapping(value = "list")
    @Log("查询api/ykt/room_device")
    @ApiOperation("查询api/ykt/room_device")
    @PreAuthorize("@el.check('dmRoomDevice:list')")
    public ResponseEntity<Object> queryList(DmRoomDeviceQueryCriteria criteria){
        return new ResponseEntity<>(dmRoomDeviceService.queryAll(criteria),HttpStatus.OK);
    }

    @GetMapping(value = "/queryRoomDeviceByUser")
    @Log("查询用户对应设备")
    @ApiOperation("查询用户对应设备")
    public ResponseEntity<Object> queryRoomDeviceByUser(DmRoomDevice criteria,Pageable pageable){
        return new ResponseEntity<>(dmRoomDeviceService.getDmRoomDeviceByUser(criteria),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增api/ykt/room_device")
    @ApiOperation("新增api/ykt/room_device")
    @PreAuthorize("@el.check('dmRoomDevice:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmRoomDevice resources){
        return new ResponseEntity<>(dmRoomDeviceService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改api/ykt/room_device")
    @ApiOperation("修改api/ykt/room_device")
    @PreAuthorize("@el.check('dmRoomDevice:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmRoomDevice resources){
        dmRoomDeviceService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除api/ykt/room_device")
    @ApiOperation("删除api/ykt/room_device")
    @PreAuthorize("@el.check('dmRoomDevice:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmRoomDeviceService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }

}