/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.ykt.service;

import me.zhengjie.ykt.domain.DmPlateNumber;
import me.zhengjie.ykt.service.dto.DmPlateNumberDto;
import me.zhengjie.ykt.service.dto.DmPlateNumberQueryCriteria;
import org.springframework.data.domain.Pageable;
import org.springframework.data.repository.query.Param;

import java.text.ParseException;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author zhirong
* @date 2021-11-10
**/
public interface DmPlateNumberService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmPlateNumberQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmPlateNumberDto>
    */
    List<DmPlateNumberDto> queryAll(DmPlateNumberQueryCriteria criteria);

    List<DmPlateNumber> yuePlateRepository();

    List<DmPlateNumber> mfPlateRepository();

    /**
     * 根据ID查询
     * @param id ID
     * @return DmPlateNumberDto
     */
    DmPlateNumberDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return DmPlateNumberDto
    */
    void create(DmPlateNumber resources);

    /**
    * APP 车牌登记
    * @param resources /
    * @return DmPlateNumberDto
    */
    void createApp(DmPlateNumber resources) throws ParseException;

    /**
    * 编辑
    * @param resources /
    */
    void update(DmPlateNumber resources);

    /**
     * 修改app的审核状态
     * @param resources /
     */
    void updateEnabled(String resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmPlateNumberDto> all, HttpServletResponse response) throws IOException;

    /**
     * 多表分页查询
     */
    Map<String,Object> getAllPlateNumber(Pageable pageable) ;

//    void getAllPlateNumber2(List<DmPlateNumberDto> all, HttpServletResponse response) throws IOException;

    /*
    搜索
     */
    List<DmPlateNumberDto> findAllBysousuo(@Param(value = "resources") DmPlateNumber resources);

    /*
    根据一卡通卡号查询
     */
    DmPlateNumber findDmPlateNumberByCardNumber(@Param(value = "card_number") Long cardNumber);

    /**
     * 编辑
     */
    String queryApprove();

    /**
     * 编辑
     */
    void updateApprove(String isApprove);

    void message(DmPlateNumber resources);

    void addYuePlate(DmPlateNumber dto);

    void deleteYuePlate(String carplate);

    void mfReserveData(DmPlateNumber dto);

    /*
    根据任务id查询
     */
    DmPlateNumber findDmPlateNumberByTaskId( String taskId);

    /**
     * 根据用户id查询
     * @param userId userId
     * @return DmPlateNumberDto
     */
    DmPlateNumber findByUserId(String userId);
}