/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.ykt.service;

import me.zhengjie.ykt.domain.DmRoomDevice;
import me.zhengjie.ykt.service.dto.DmRoomDeviceDto;
import me.zhengjie.ykt.service.dto.DmRoomDeviceQueryCriteria;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author zhirong
* @date 2021-11-19
**/
public interface DmRoomDeviceService {

    /**
     * 导入excel数据
     * @param file
     * @return void
     *
     */
    List<Map<Object, Object>> importExcel(MultipartFile file);

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmRoomDeviceQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmRoomDeviceDto>
    */
    List<DmRoomDeviceDto> queryAll(DmRoomDeviceQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmRoomDeviceDto
     */
    DmRoomDeviceDto findById(Long id);

    List<DmRoomDevice> getDmRoomDeviceByUser(DmRoomDevice dmRoomDevice);

    /**
    * 创建
    * @param resources /
    * @return DmRoomDeviceDto
    */
    DmRoomDeviceDto create(DmRoomDevice resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmRoomDevice resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmRoomDeviceDto> all, HttpServletResponse response) throws IOException;

    /**
     * 根据CODE查询
     * @param code ID
     * @return DmRoomDevice
     */
    List<DmRoomDevice> findByCode(String code,String doorId);

    /**
     * 根据CODE查询
     * @param code ID
     * @return DmRoomDevice
     */
    List<DmRoomDevice> findByCode(String code);

}